@@ -213,11 +213,21 @@ def run(command, verbose, debug, dry_run, no_lock, update,
 @click.option('--limit-parallelism', '-l', type=int, default=None,
     help='Limit the parallelism to the given number, unlimitted if omitted',
     )
+@click.option('--include-dir', '-I',
+    multiple=True, default=[],
+    help='A glob of a directory that needs to be included, this option can be used multiple times',
+    show_default=True
+    )
+@click.option('--exclude-dir', '-E',
+    multiple=True, default=[],
+    help='A glob of a directory that needs to be excluded, this option can be used multiple times',
+    show_default=True
+    )
 @click.argument('terragrunt-args', nargs=-1, type=click.UNPROCESSED)
 @click.version_option(version=__version__)
 def run_all(command, verbose, debug, dry_run, no_lock, update, exclude_external_dependencies,
     step_by_step, planfile, auto_approve, working_dir, start_at_step,
-    limit_parallelism, terragrunt_args):
+    limit_parallelism, include_dir, exclude_dir, terragrunt_args):
     """ Executes a terragrunt command across multiple projects """
 
     check_latest_version(verbose)
@@ -236,6 +246,8 @@ def run_all(command, verbose, debug, dry_run, no_lock, update, exclude_external_
         working_dir=working_dir,
         start_at_step=start_at_step,
         limit_parallelism=limit_parallelism,
+        include_dir=include_dir,
+        exclude_dir=exclude_dir,
         terragrunt_args=terragrunt_args,
     )
 
@@ -615,12 +627,22 @@ def deploy(
     help='Whether or not external dependencies must be ignored',
     show_default=True
     )
+@click.option('--include-dir', '-I',
+    multiple=True, default=[],
+    help='A glob of a directory that needs to be included, this option can be used multiple times',
+    show_default=True
+    )
+@click.option('--exclude-dir', '-E',
+    multiple=True, default=[],
+    help='A glob of a directory that needs to be excluded, this option can be used multiple times',
+    show_default=True
+    )
 @click.option('--working-dir', '-w', default=None,
     help='Working directory, when omitted the current directory is used',
     )
 @click.argument('terragrunt-args', nargs=-1, type=click.UNPROCESSED)
 @click.version_option(version=__version__)
-def show_graph(verbose, backwards, exclude_external_dependencies, working_dir, terragrunt_args):
+def show_graph(verbose, backwards, exclude_external_dependencies, working_dir, include_dir, exclude_dir, terragrunt_args):
     """ Shows the dependencies of a project """
 
     check_latest_version(verbose)
@@ -630,6 +652,8 @@ def show_graph(verbose, backwards, exclude_external_dependencies, working_dir, t
         backwards=backwards,
         exclude_external_dependencies=exclude_external_dependencies,
         working_dir=working_dir,
+        include_dir=include_dir,
+        exclude_dir=exclude_dir,
         terragrunt_args=terragrunt_args,
     )
 
