@@ -17,6 +17,7 @@ import sys
 import subprocess
 import shlex
 import shutil
+import glob
 import re
 import tempfile
 import json
@@ -25,7 +26,6 @@ import threading
 import queue
 import multiprocessing
 import click
-import hcl
 import networkx as nx
 
 from .printer import Printer
@@ -62,18 +62,19 @@ class TgWrap():
 
     def _construct_command(self, command, debug, exclude_external_dependencies,
         non_interactive=True, no_auto_approve=True, no_lock=True, update=False,
-        planfile=None, working_dir=None, limit_parallelism=None, terragrunt_args=()):
+        planfile=None, working_dir=None, limit_parallelism=None, 
+        include_dir=[], exclude_dir=[], terragrunt_args=()):
         """ Constructs the command """
 
         PLANFILE_NAME="planfile"
 
         commands = {
-            'generic': '{base_command} {command} --terragrunt-non-interactive {no_auto_approve} {ignore_deps} {debug_level} {update} {working_dir} {parallelism} {terragrunt_args}',
-            'info': '{base_command} terragrunt-info --terragrunt-non-interactive {ignore_deps} {debug_level} {update} {working_dir} {terragrunt_args}',
-            'plan': '{base_command} {command} --terragrunt-non-interactive  -out={planfile_name} {ignore_deps} {debug_level} {lock_level} {update} {working_dir} {parallelism} {terragrunt_args}',
-            'apply': '{base_command} {command} {non_interactive} {no_auto_approve} {parallelism} {ignore_deps} {debug_level} {update} {working_dir} {parallelism} {terragrunt_args} {planfile}',
+            'generic': '{base_command} {command} --terragrunt-non-interactive {no_auto_approve} {update} {parallelism} {common}',
+            'info': '{base_command} terragrunt-info --terragrunt-non-interactive {update} {common}',
+            'plan': '{base_command} {command} --terragrunt-non-interactive  -out={planfile_name} {lock_level} {update} {parallelism} {common}',
+            'apply': '{base_command} {command} {non_interactive} {no_auto_approve} {update} {parallelism} {common} {planfile}',
             'show': '{base_command} {command} --terragrunt-non-interactive {ignore_deps} {update} -json {planfile_name}', # no working dir allowed here!!!
-            'destroy': '{base_command} {command} {non_interactive} {no_auto_approve} {ignore_deps} {debug_level} {working_dir} {parallelism} {terragrunt_args} {planfile}',
+            'destroy': '{base_command} {command} {non_interactive} {no_auto_approve} {parallelism} {common} {planfile}',
         }
 
         lock_stmt         = '-lock=false' if no_lock else ''
@@ -85,8 +86,11 @@ class TgWrap():
         working_dir_stmt  = f'--terragrunt-working-dir {working_dir}' if working_dir else ''
         parallelism_stmt  = f'--terragrunt-parallelism {limit_parallelism}' if limit_parallelism else ''
         planfile_stmt     = f'{PLANFILE_NAME}' if planfile else ''
+        include_dir_stmt  = f'--terragrunt-strict-include  --terragrunt-include-dir {" --terragrunt-include-dir ".join(include_dir)}' if len(include_dir) > 0 else ""
+        exclude_dir_stmt  = f'--terragrunt-exclude-dir {" --terragrunt-exclude-dir ".join(exclude_dir)}' if len(exclude_dir) > 0 else ""
 
         base_command      = 'terragrunt run-all'
+        common_commands = f"{ignore_deps_stmt} {debug_stmt} {working_dir_stmt} {include_dir_stmt} {exclude_dir_stmt} {' '.join(terragrunt_args)}"
 
         # if we are using an 'init' together with update, add the `-upgrade` as well
         if command.lower() == 'init' and update:
@@ -99,14 +103,12 @@ class TgWrap():
                 lock_level=lock_stmt,
                 update=update_stmt,
                 ignore_deps=ignore_deps_stmt,
-                debug_level=debug_stmt,
                 no_auto_approve=auto_approve_stmt,
                 non_interactive=interactive_stmt,
-                working_dir=working_dir_stmt,
                 parallelism=parallelism_stmt,
                 planfile=planfile_stmt,
+                common=common_commands,
                 planfile_name=PLANFILE_NAME,
-                terragrunt_args=' '.join(terragrunt_args),
             )
         else:
             full_command = commands.get(command, commands.get('generic'))
@@ -118,12 +120,14 @@ class TgWrap():
 
         return full_command
 
-    def _prepare_groups(self, graph, exclude_external_dependencies, working_dir):
+    def _prepare_groups(self, graph, exclude_external_dependencies, working_dir,
+                        exclude_dir=[], include_dir=[]):
         """ Prepare the list of groups that will be executed """
 
         working_dir = os.path.abspath(working_dir) if working_dir else os.getcwd()
         self.printer.verbose(f"Check for working dir: {working_dir}")
 
+        print(include_dir, exclude_dir)
         groups = []
         for group in nx.topological_generations(graph):
             try:
@@ -132,12 +136,37 @@ class TgWrap():
                 pass
 
             for idx, directory in enumerate(group):
+                dir_excluded = False
+                dir_included = True if len(include_dir) == 0 else False
+                dir_excluded_reason = ""
+
                 common_path = os.path.commonpath([working_dir, os.path.abspath(directory)])
                 # self.printer.verbose(f'Common path for dir {directory}: {common_path}')
                 if common_path != working_dir \
                     and exclude_external_dependencies:
+                    dir_excluded = True
+                    dir_excluded_reason = "directory out of scope"
+
+                for i in exclude_dir:
+                    exclude_paths = glob.glob(i)
+                    if directory in exclude_paths:
+                        dir_excluded = True
+                        dir_excluded_reason = "directory explicitly excluded"
+
+                # if we have a specific set of include_dirs, then everything else should be excluded
+                for i in include_dir:
+                    include_paths = glob.glob(i)
+                    if directory in include_paths:
+                        dir_included = True
+
+                if dir_excluded:
                     self.printer.verbose(
-                        f"- Remove directory from group as it falls out of scope: {directory}"
+                        f"- Remove directory '{directory}' from group: {dir_excluded_reason}"
+                        )
+                    group[idx] = None
+                elif not dir_included:
+                    self.printer.verbose(
+                        f"- Remove directory '{directory}' from group: specific list of include dirs given"
                         )
                     group[idx] = None
                 else:
@@ -330,7 +359,7 @@ class TgWrap():
     def _run_di_graph(
         self, command, exclude_external_dependencies, start_at_step, dry_run,
         parallel_execution=False, ask_for_confirmation=False, collect_output_file=None,
-        backwards=False, working_dir=None,
+        backwards=False, working_dir=None, include_dir=[], exclude_dir=[], 
         ):
         "Runs the desired command in the directories as defined in the directed graph"
 
@@ -341,6 +370,8 @@ class TgWrap():
             graph=graph,
             exclude_external_dependencies=exclude_external_dependencies,
             working_dir=working_dir,
+            include_dir=include_dir,
+            exclude_dir=exclude_dir,
             )
 
         if not groups:
@@ -379,7 +410,7 @@ class TgWrap():
                 module_nbr=idx2+1
                 progress = f'module {module_nbr} (of {nbr_of_modules}) of group {group_nbr} (of {nbr_of_groups})'
 
-                step_nbr = group_nbr + module_nbr/10
+                step_nbr = group_nbr + module_nbr/100
                 if step_nbr < start_at_step:
                     self.printer.normal(f'Skip step {step_nbr}, start at {start_at_step}')
                     continue
@@ -412,9 +443,9 @@ class TgWrap():
                     )
 
                     if stop_processing:
-                        self.printer.warning(f"Processing needs to be stopped at step {group_nbr}.{module_nbr}.")
+                        self.printer.warning(f"Processing needs to be stopped at step {step_nbr}.")
                         self.printer.normal(
-                            f"After you've fixed the problem, you can continue where you were left by adding '--start-at-step {group_nbr}.{module_nbr}'."
+                            f"After you've fixed the problem, you can continue where you were left by adding '--start-at-step {step_nbr}'."
                             )
                         sys.exit(1)
 
@@ -528,7 +559,7 @@ class TgWrap():
 
     def run_all(self, command, debug, dry_run, no_lock, update,
         exclude_external_dependencies, step_by_step, planfile, auto_approve, 
-        working_dir, start_at_step, limit_parallelism, terragrunt_args):
+        working_dir, start_at_step, limit_parallelism, include_dir, exclude_dir, terragrunt_args):
         """ Executes a terragrunt command across multiple modules """
 
         self.printer.verbose(f"Attempting to execute 'run-all {command}'")
@@ -551,6 +582,8 @@ class TgWrap():
             working_dir=None if step_by_step else working_dir,
             terragrunt_args=terragrunt_args,
             limit_parallelism=limit_parallelism,
+            include_dir=[] if step_by_step else include_dir,
+            exclude_dir=[] if step_by_step else exclude_dir,
         )
 
         if step_by_step:
@@ -565,7 +598,9 @@ class TgWrap():
                 ask_for_confirmation=(not auto_approve),
                 dry_run=dry_run,
                 start_at_step=start_at_step,
-                backwards=True if command.lower() in ['destroy'] else False
+                backwards=True if command.lower() in ['destroy'] else False,
+                include_dir=include_dir,
+                exclude_dir=exclude_dir,
             )
         else:
             if dry_run:
@@ -965,7 +1000,7 @@ locals {{
             except Exception:
                 pass
 
-    def show_graph(self, backwards, exclude_external_dependencies, working_dir, terragrunt_args):
+    def show_graph(self, backwards, exclude_external_dependencies, working_dir, include_dir, exclude_dir, terragrunt_args):
         """ Shows the dependencies of a project """
 
         self.printer.verbose(f"Attempting to show dependencies")
@@ -980,6 +1015,8 @@ locals {{
             graph=graph,
             exclude_external_dependencies=exclude_external_dependencies,
             working_dir=working_dir,
+            include_dir=include_dir,
+            exclude_dir=exclude_dir,
             )
 
         self.printer.header("The following groups will be processed:")
